;******************************************************************************
; File Name:	 ch7_e1.asm
; Target System: C240x Evaluation Board
; Description:	DC MOTOR CLOCKWISE ROTATION SCHEME 1: Apply PWM on only one
;				of the four power switches. These four switches are driven by
;				I/O/TxPWM pins:
;					IOPB2 - T1;	IOPB3 - T2; IOPB4/T1PWM - T3; IOPB5/T2PWM - T4
;				PWM frequency is 5 kHz
;
; Date:			01/11/2004
; Author:		Min Dai
;******************************************************************************

;******************************************************************************
;					  SYSTEM OPTIONS
;******************************************************************************
real_time	.set	0      		; 1 for real time mode, otherwise set 0
clockwise	.set	1			; 1=clockwise, 0=counterclockwise
per_val		.set	6000		;Value to be loaded in the GPT period register
								;30000000/1/5000=6000
cmpr_val	.set	3000		;Value to be loaded in the GPT compare register

;******************************************************************************

;-----------------------------------------------------------------------------
; External references
;-----------------------------------------------------------------------------
		.include 	"f2407.h"
;		.global 	MON_RT_CNFG

		.ref		SYS_INIT

;-----------------------------------------------------------------------------
; Local Variable Declarations
;-----------------------------------------------------------------------------
		.def		GPR0				;General purpose register.
        
		.bss		GPR0,1				;General purpose register. 
		
;==============================================================================
; V E C T O R    T A B L E    ( including RT monitor traps )
;==============================================================================
;        .include 	"c200mnrt.i" 		; Include conditional assembly options.

		.global		_c_int0,PHANTOM,GISR1,GISR2,GISR3,GISR4,GISR5,GISR6

;==============================================================================
; M A I N   C O D E  - starts here
;==============================================================================
		.text
_c_int0:
		CALL		SYS_INIT                ;DSP initialization

;----------------------------------------------------------
; Initialise the Real time monitor
;----------------------------------------------------------

		POINT_PG0

;---Real Time option---------------
	.if (real_time)
		CALL		MON_RT_CNFG				;For Real-Time
	.endif
;----------------------------------

;----------------------------------------------------------
; System Interrupt Init.
;----------------------------------------------------------

;---Real Time option --------------------------------------------------
	.if (real_time)
		SPLK	#0000000001000010b,IMR	;En Int lvl 2 & 7 for T1 and RT  
		        ;|||||||||!||||||
			    ;5432109876543210
	.endif


	.if (real_time != 1)
		SPLK	#0000000000000010b,IMR	;Enable Int 2 only for T1
			    ;||||!!!!||||!!!!		
			    ;5432109876543210
	.endif

		SPLK	#0FFFFh, IFR			;Clear any pending Ints
;		EINT							;Enable global Ints
;-----------------------------------------------------------------------
;-------------------------------------------------------------------------
;		RESET SECTION - BEGINS
;-------------------------------------------------------------------------
        LDP     #DP_EVA                 ;set data page
        SPLK    #0FFFFh, EVAIFRA        ;clear all EVA group A interrupts
        SPLK    #0FFFFh, EVAIFRB        ;clear all EVA group B interrupts
        SPLK    #0FFFFh, EVAIFRC        ;clear all EVA group C interrupts
        SPLK    #00000h, EVAIMRA        ;enabled desired EVA group A interrupts
        SPLK    #00000h, EVAIMRB        ;enabled desired EVA group B interrupts
        SPLK    #00000h, EVAIMRC        ;enabled desired EVA group C interrupts

        LDP		#DP_EVA                 ;set data page
		SPLK   	#0, GPTCONA
		SPLK   	#0, T1CON
		SPLK   	#0, T2CON

		SPLK   	#0, COMCONA
		SPLK   	#0, ACTRA
		SPLK   	#0, DBTCONA
		SPLK   	#0, CAPCONA

;-------------------------------------------------------------------------
;		RESET SECTION - ENDS
;-------------------------------------------------------------------------
		
		LDP		#DP_PF2
		SPLK	#0000h, MCRA	;Set pins to be I/O
		SPLK	#3C3Ch,PBDATDIR	;Turn off T1, T2, T3 and T4 by 
								;outputting 1 on all pins.

	  .if clockwise=0
		SPLK	#1000h, MCRA	;Configure the o/p pins for T1PWM(IOPB4),
 								;IOPB2, IOPB3, IOPB5. i.e. output PWM 
								;at input of T3.
		SPLK	#2C28h,PBDATDIR	;Turn on T1	

		LDP    	#DP_EVA
		SPLK	#0045h,GPTCONA	;Enable Compare outputs of all GPTs
								;GPT1 and 2 compare outputs - active low
		SPLK	#cmpr_val,T1CMPR;Load compare register
		SPLK    #per_val, T1PR	;Load period register
		SPLK    #1042h, T1CON	;Select continuous up counting mode
								;Prescaler = 1, enable timer compare
								;operation, enable timer operation.
	  .else
		SPLK	#2000h, MCRA	;Configure the o/p pins for T2PWM(IOPB5),
 								;IOPB2, IOPB3, IOPB4. i.e. output PWM 
								;at input of T3.


		SPLK	#1C14h,PBDATDIR	;Turn on T2	
		
		LDP    	#DP_EVA
		SPLK	#0045h,GPTCONA	;Enable Compare outputs of all GPTs
								;GPT1 and 2 compare outputs - active low
		SPLK	#cmpr_val,T2CMPR;Load compare register
		SPLK    #per_val, T2PR	;Load period register
		SPLK    #1042h, T2CON	;Select continuous up counting mode
								;Prescaler = 1, enable timer compare
								;operation, enable timer operation.
	  .endif
	  					
END		B       END				;End of program


;==============================================================================
; I S R  -  PHANTOM
;
; Description:	Dummy ISR, used to trap spurious interrupts.
;
; Modifies:
;
; Last Update:	16-06-95
;==============================================================================
PHANTOM B	PHANTOM
GISR1 	RET
GISR2 	RET
GISR3 	RET
GISR4 	RET
GISR5 	RET
GISR6	RET