;******************************************************************************
; File Name:	 ch7_e2.asm
; Target System: C240x Evaluation Board
; Description:	DC MOTOR CLOCKWISE ROTATION SCHEME 2
;				PWM frequency is 1 kHz, duty cycle is 50%
;				Capture the encoder pulses and calculate for the speed -
;				 	the incremental encoder outputs 2950 pulses/revolution
;				The four power switches are driven by I/O pins:
;					IOPB0 - T1;	IOPB1 - T2; IOPB2 - T3; IOPB3 - T4
;
; Date:			01/12/2004
; Author:		Min Dai
;******************************************************************************

;******************************************************************************
;					  SYSTEM OPTIONS
;******************************************************************************
real_time	.set	0      		; 1 for real time mode, otherwise set 0
STATE1		.set	0F0Ah
STATE2		.set	0F06h
;STATE2		.set	0F0Ah
STATE3		.set	0F0Ah
STATE4		.set	0F09h
T1PERIOD	.set	234        	; 30000000/32/4000 = 234 to make PWM freq=1 kHz

;******************************************************************************

;-----------------------------------------------------------------------------
; External references
;-----------------------------------------------------------------------------
		.include 	"f2407.h"
;		.global 	MON_RT_CNFG

		.ref		SYS_INIT

;-----------------------------------------------------------------------------
; Local Variable Declarations
;-----------------------------------------------------------------------------
		.def		GPR0				;General purpose register.
        
		.bss		GPR0,1				;General purpose register. 
		.bss		ENCDR_CNT,1			;Encoder pulse counter
		.bss		SEC_CTR,1			;Second counter
		.bss		SPEED,1				;Value of speed
		.bss		CTR,1				;State counter
		
;==============================================================================
; V E C T O R    T A B L E    ( including RT monitor traps )
;==============================================================================
;        .include 	"c200mnrt.i" 		; Include conditional assembly options.

		.global		_c_int0,PHANTOM,GISR1,GISR2,GISR3,GISR4,GISR5,GISR6

;==============================================================================
; M A I N   C O D E  - starts here
;==============================================================================
		.text
_c_int0:
		CALL		SYS_INIT            ;DSP initialization

;----------------------------------------------------------
; Initialise the Real time monitor
;----------------------------------------------------------

		POINT_PG0

;---Real Time option---------------
	.if (real_time)
		CALL		MON_RT_CNFG			;For Real-Time
	.endif
;----------------------------------

;----------------------------------------------------------
; System Interrupt Init.
;----------------------------------------------------------

;---Real Time option --------------------------------------------------
	.if (real_time)
		SPLK	#0000000001001010b,IMR	;En Int lvl 2, 4 & 7   
		        ;|||||||||!||||||
			    ;5432109876543210
	.endif


	.if (real_time != 1)
		SPLK	#0000000000001010b,IMR	;Enable Int 2 & Int 4
			    ;||||!!!!||||!!!!		
			    ;5432109876543210
	.endif

		SPLK	#0FFFFh, IFR			;Clear any pending Ints
;		EINT							;Enable global Ints
;-----------------------------------------------------------------------
;-------------------------------------------------------------------------
;		RESET SECTION - BEGINS
;-------------------------------------------------------------------------
        LDP     #DP_EVA                 ;set data page
        SPLK    #0FFFFh, EVAIFRA        ;clear all EVA group A interrupts
        SPLK    #0FFFFh, EVAIFRB        ;clear all EVA group B interrupts
        SPLK    #0FFFFh, EVAIFRC        ;clear all EVA group C interrupts
        SPLK    #00000h, EVAIMRA        ;enabled desired EVA group A interrupts
        SPLK    #00000h, EVAIMRB        ;enabled desired EVA group B interrupts
        SPLK    #00000h, EVAIMRC        ;enabled desired EVA group C interrupts

        LDP		#DP_EVA                 ;set data page
		SPLK   	#0, GPTCONA             ;Reset EVA registers
		SPLK   	#0, T1CON
		SPLK   	#0, T2CON

		SPLK   	#0, COMCONA
		SPLK   	#0, ACTRA
		SPLK   	#0, DBTCONA
		SPLK   	#0, CAPCONA

		LDP		#ENCDR_CNT
		SPLK	#0, ENCDR_CNT   		;Reset variables 
		SPLK	#0, CTR
		SPLK	#0, SEC_CTR
		SPLK	#0, SPEED
;-------------------------------------------------------------------------
;		RESET SECTION - ENDS
;-------------------------------------------------------------------------


		LDP		#DP_PF2
		SPLK	#0008h, MCRA			;Select IOB0-3 as outputs
										;Select CAP1 function of pin
		
		LDP		#DP_EVA
		SPLK	#0055h, GPTCONA
		SPLK	#T1PERIOD, T1PR
		SPLK	#0, T1CNT
		SPLK	#0, T2CNT
 
		SPLK	#150Ah, T1CON		;SET PRESCALER =32, continuous up counting mode,
									;Disable timer, disable timer compare operation

		SPLK   	#02080h, CAPCONA	;Enable capture units 1 and 2
									;Select GP Timer 2 as time base for capture unit 1
									;This time base is not used in this program
									;Detect falling edge
		SPLK   	#00FFh, CAPFIFOA    ;Clear the capture unit FIFO initially
		SPLK   	#0080h, EVAIMRA     ;Enable timer1 period interrupt
		SPLK   	#0001h, EVAIMRC     ;Enable capture unit 1 interrupt
		
		SBIT1	T1CON, 	0000000001000000b	;Enable timer 1 by setting Bit6
					;	||||||||||||||||
					;	5432109876543210
		CLRC	INTM

END		B	END

;=====================================================================
; GISR2  -  	Timer interrupt service routine
; Description:	Outputs the proper sequence for all the transistors.
;				Loads the number of pulses in SPEED at the end of
;				every 0.1 sec interval.
; Modifies:		SPEED, ENCDR_CNT, SEC_CTR
;=====================================================================
GISR2	LDP		#SEC_CTR
		LACC	SEC_CTR
		ADD		#1
		SUB		#400
		BCND	NXT, NEQ			;Check if 0.1 sec
		LACC	ENCDR_CNT			;If 0.1 sec then SPEED=ENCDR_CNT
		SACL	SPEED
		SPLK	#0, ENCDR_CNT		;Reset ENCDR_CNT and SEC_CTR
		SPLK	#0, SEC_CTR
		B		NXT1
NXT		LACC	SEC_CTR				;If not 1 sec increment SEC_CTR
		ADD		#1
		SACL	SEC_CTR
NXT1	LACC	CTR
		BCND	ST2, NEQ			;Check the state number if 0
		LDP		#DP_PF2
    	SPLK	#STATE1, PBDATDIR	;Output sequence for STATE1
		LDP		#CTR
		SPLK	#1,CTR				;Update state counter
		B		FIN
ST2		SUB		#1
		BCND	ST3,NEQ				;Check the state number if 1
		LDP		#DP_PF2
  		SPLK	#STATE2, PBDATDIR	;Output sequence for STATE2
		LDP		#CTR
		SPLK	#2,CTR				;Update state counter
		B		FIN
ST3		SUB		#1
		BCND	ST4,NEQ				;Check the state number if 2
		LDP		#DP_PF2
		SPLK	#STATE3, PBDATDIR	;Output sequence for STATE3
		LDP		#CTR
		SPLK	#3,CTR				;Update state counter
		B		FIN
ST4		LDP		#DP_PF2
		SPLK	#STATE4, PBDATDIR	;Output sequence for STATE4
		LDP		#CTR
		SPLK	#0, CTR				;Reset state counter
FIN		LDP		#DP_EVA				;Set data page   
        SPLK    #0FFFFh, EVAIFRA    ;clear all EVA group A interrupts
		CLRC	INTM				;Enable maskable interrupts
		RET

;=====================================================================
; GISR4  -  Capture Unit interrupt service routine
; Description:	Counts the number of pulses from encoder.
; Modifies:	ENCDR_CNT
;=====================================================================
GISR4	LDP		#ENCDR_CNT
		LACC	ENCDR_CNT
		ADD		#1
		SACL	ENCDR_CNT			;Increment the pulse counter
		LDP		#DP_EVA
        SPLK    #0FFFFh, EVAIFRC	;clear all EVA group C interrupts
		CLRC	INTM				;Enable maskable interrupts
		RET

;==============================================================================
; I S R  -  PHANTOM
;
; Description:	Dummy ISR, used to trap spurious interrupts.
;
; Modifies:
;
; Last Update:	16-06-95
;==============================================================================
PHANTOM B		PHANTOM
GISR1 	RET
;GISR2 	RET
GISR3 	RET
;GISR4 	RET
GISR5 	RET
GISR6	RET